Package: tsdf
Type: Package
Title: Two-/Three-Stage Designs for Phase 1&2 Clinical Trials
Version: 1.1-9
Date: 2026-04-25
Authors@R: c(
    person("Wenchuan", "Guo", email = "wguo1017@gmail.com", role = c("aut", "cre")),
    person("Jianan", "Hui", email = "jiananhuistat@gmail.com", role = "aut"),
    person("Bob", "Zhong", role = "aut")
  )
Description: Calculates Zhong's optimal two-/three-stage Phase II designs
    for single-arm trials, generates target-toxicity decision tables for
    two-/three-stage Phase I dose-finding, and supports dose-finding
    simulations using custom decision tables. The Phase II design is based on
    Zhong (2012) <doi:10.1016/j.cct.2012.07.006>.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-25 22:45:03 UTC; wguo
Author: Wenchuan Guo [aut, cre],
  Jianan Hui [aut],
  Bob Zhong [aut]
Maintainer: Wenchuan Guo <wguo1017@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-26 00:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 20:32:00 UTC; unix
