# Generated by roxygen2: do not edit by hand

S3method(plot,dec.sim)
S3method(plot,dec.table)
S3method(print,dec.table)
S3method(print,opt.design)
S3method(print,summary.dec.sim)
S3method(summary,dec.sim)
export(adj.two)
export(dec.sim)
export(dec.table)
export(opt.design)
export(sl.sim)
import(graphics)
import(stats)
import(utils)
