tsdf
===

[![](https://www.r-pkg.org/badges/version/tsdf)](https://cran.r-project.org/package=tsdf)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/tsdf)](https://cran.r-project.org/package=tsdf)

### What is it?
`tsdf` calculates optimal Zhong's two-/three-stage Phase II designs, generates decision tables for two-/three-stage Phase I dose-finding, and allows users to run dose-finding simulations with custom decision tables.
### Installation
To install from `CRAN`:
```r
install.packages("tsdf")
```

### References
[1] Bob Zhong, Single-arm phase IIA clinical trials with go/no-go decisions, Contemp Clin Trials, 2012

[2] Wenchuan Guo and Bob Zhong, Target toxicity design for phase I dose-finding, Statistical Theory and Related Fields, 2021

