% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{plot.dec.table}
\alias{plot.dec.table}
\title{Plot a decision table from a "dec.table" object}
\usage{
\method{plot}{dec.table}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"dec.table"}, typically returned by
\code{\link{dec.table}}.}

\item{...}{Unused arguments.}
}
\description{
\code{plot} method for class \code{"dec.table"}.
}
\details{
\code{plot.dec.table} plots the decision boundaries.
}
\examples{
truep <- c(0.3, 0.45, 0.5, 0.6)
out <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
plot(out)
}
