% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{summary.dec.sim}
\alias{summary.dec.sim}
\title{Summarize simulation results from a "dec.sim" object}
\usage{
\method{summary}{dec.sim}(object, pt, ...)
}
\arguments{
\item{object}{An object of class \code{"dec.sim"}, returned by
\code{\link{dec.sim}} or \code{\link{sl.sim}}.}

\item{pt}{Target toxicity for each scenario.}

\item{...}{Unused arguments.}
}
\description{
\code{summary} method for class \code{"dec.sim"}.
}
\details{
\code{summary} formats key statistics from dose-finding
  simulations. Given the target toxicity, it reports the probability of
  selecting each dose level as the MTD, the probability of selecting doses
  above the true MTD, the probability of selecting the true MTD, and the
  probability that subjects are treated at or below the true MTD. The true
  MTD is defined as the highest dose level with toxicity probability less
  than or equal to the target toxicity. If the target is below the smallest
  toxicity probability, the lowest dose level is treated as the MTD. For
  example, if the target is 0.3 and the true toxicity probabilities for five
  doses are 0.1, 0.25, 0.35, 0.40, and 0.50, then the MTD is dose 2.
}
\examples{
test.file <- system.file("extdata", "testS.csv", package = "tsdf")
dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
out <- sl.sim(dt$table, test.file)
pt <- c(0.3, 0.4)
summary(out, pt)
}
